/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * Contributor(s): Eric Murphy   , ericmurphy@jabber.org
 *                 Michael Hearn , tweedledee@jabber.org
 *                 Jim Ray       , jray@lanscape.net
 *                 Jan Hecking   , jhecking@netgaroo.com
 * 2002.11.24
 *
 *	Change Log:
 *
 *		1. Move back 3 files from chrome://jabberzilla/content/JabberXMHandlers: 
 *				message.js, iq.js, presence.js (by Patton.gao@sun.com at Dec 28 2002)
 *
 *		2. Fixed Bug 2796 (Patton.gao@sun.com at Dec 29 2002)
 *
 **********************************************************************************/

const CID         = Components.ID("{a4cdcfae-1dd1-11b2-9c25-9b5d19103742}");
const CiJabberXM  = Components.interfaces.JabberXM;
const nsISupports = Components.interfaces.nsISupports;

/******* SECTION: JABBERXM CONSTRUCTOR *******/

function JabberXM(){}

function initialize()
{
  // This initializes the subcomponents, and these mostly handle incoming data.
  if (JabberXMModule.socket==null)
  {
    jabberXMSax =
      new Components.Constructor("@jabberxm.mozdev.org/sax;1","JabberXMSax");

    JabberXMModule.stream = new jabberXMSax();
    JabberXMModule.stream.setDocumentHandler(new Handler());

    jabberXMSocket =
      new Components.Constructor("@jabberxm.mozdev.org/socket;1","JabberXMSocket");

    JabberXMModule.socket = new jabberXMSocket();
    JabberXMModule.socket.stream(JabberXMModule.stream);
  }
}

/******* SECTION: HANDLERS *******/

JabberXM.prototype.setHandlers = function(session)
{
  // This initializes the components code
  JabberXMModule.session  = session.wrappedJSObject;
  
  JabberXMModule.message  = new Message();
}

function Handler()
{
  this.wrappedJSObject = this;

  // Separates out the different packet types
  this.packetHandler = function(packet)
  {
		if(packet.name == "message")
      JabberXMModule.message.packetMessage(packet);

    else if (packet.name == "iq")
      JabberXMModule.iq.packetIQ(packet);

    else if (packet.name == "presence")
      JabberXMModule.presence.packetPresence(packet);

    else if (packet.name == "stream:stream")
      JabberXMModule.session.packetSession(packet);
  }
}

/******* SECTION: XML *******/

JabberXM.prototype.sendXML = function(xml)
{
  if (xml != null || xml != '')
  {
    JabberXMModule.socket.write(xml);
    return true;
  }
  else
    return false;
}

JabberXM.prototype.setViewStream = function(viewStream)
{
  viewStream = viewStream.wrappedJSObject;
  JabberXMModule.socket.setViewStream(viewStream);
}

/******* SECTION: SESSION *******/

JabberXM.prototype.getUsername = function()
{
  return(JabberXMModule.username);
}

JabberXM.prototype.getServer = function()
{
  return(JabberXMModule.server);
}

JabberXM.prototype.getJID = function()
{
  return(JabberXMModule.username + '@' + JabberXMModule.server);
}

JabberXM.prototype.getPassword = function()
{
  return(JabberXMModule.password);
}

JabberXM.prototype.setRegister = function(username,password)
{
  JabberXMModule.username = username;
  JabberXMModule.password = password;
  JabberXMModule.doAuth = true;
}

JabberXM.prototype.isAuth = function()
{
  return JabberXMModule.doAuth;
}

JabberXM.prototype.register = function()
{
  JabberXMModule.socket.write
  (
    '<iq type="set" id="register"><query xmlns="jabber:iq:register">'
    + '<username>' + JabberXMModule.username + '</username>' 
    + '<password>' + JabberXMModule.password + '</password>'
    + '</query></iq>'
  );
}

JabberXM.prototype.setAuthenticate = function(username,password,resource,priority)
{
  if(resource=="" || resource==null)
    this.resource = ("jabberzilla");

  if(priority=="" || priority==null)
    this.priority = 0;

  JabberXMModule.username = username;
  JabberXMModule.password = password;
  JabberXMModule.resource = resource;
  JabberXMModule.priority = priority;
  JabberXMModule.doAuth   = false;
}

JabberXM.prototype.authenticate = function(type,passkey)
{
	if(type=="digest")
  {
    JabberXMModule.socket.write
    (
      "<iq type='set' id='auth'><query xmlns='jabber:iq:auth'>" + 
      "<username>" + JabberXMModule.username + "</username>" + 
      "<digest>" + passkey + "</digest>" + 
      "<priority>" + JabberXMModule.priority + "</priority>" +
      "<resource>" + JabberXMModule.resource + "</resource>" + 
      "</query></iq>"
    );
  }
  else if(type=="normal")
  {
    JabberXMModule.socket.write
    (
      "<iq type='set' id='auth'><query xmlns='jabber:iq:auth'>" + 
      "<username>" + JabberXMModule.username + "</username>" + 
      "<password>" + passkey + "</password>" + 
      "<priority>" + JabberXMModule.priority + "</priority>" +
      "<resource>" + JabberXMModule.resource + "</resource>" + 
      "</query></iq>"
    );
  }
  JabberXMModule.socket.write("<presence/>");
  JabberXMModule.isConnected = true;
}

JabberXM.prototype.connect = function(socketTypeCount, socketTypes, host, port, proxyInfo)
{
  if (JabberXMModule.isConnected)
  {
    return false;
  }
  else
  {
    JabberXMModule.socket.open(socketTypeCount,socketTypes,host,port,proxyInfo);
		JabberXMModule.socket.initAsyncRead();

  	JabberXMModule.socket.write
  	(
 	   "<stream:stream to='" + host
	   + "' xmlns:stream='http://etherx.jabber.org/streams'"
 	   + "  xmlns='jabber:client'>"
 	  );
 	  JabberXMModule.server = host;
   	return true;
  }
}

JabberXM.prototype.isConnected = function()
{
  return JabberXMModule.isConnected;
}

JabberXM.prototype.disconnect = function()
{
  if(JabberXMModule.isConnected)
    JabberXMModule.socket.write("<presence type='unavailable'/></stream:stream>");

  JabberXMModule.socket.close();

  JabberXMModule.isConnected = false;
  JabberXMModule.hasReceivedRoster = false;

  JabberXMModule.presence = null;
  JabberXMModule.server   = '';
  JabberXMModule.username = '';

  JabberXMModule.iq = new IQ();
	JabberXMModule.presence = new Presence();	

	JabberXMModule.chatWindows = new Object();
  JabberXMModule.messageWindows = new Object();

  JabberXMModule.socket = null;
  initialize();
}

JabberXM.prototype.sessionTimeLength = function()
{
  // Do a time comparison
}

JabberXM.prototype.changePassword = function(oldPassword,newPassword)
{
}

/******* SECTION: ROSTER *******/

JabberXM.prototype.hasReceivedRoster = function()
{
  return JabberXMModule.hasReceivedRoster;
}

JabberXM.prototype.requestRoster = function()
{
  JabberXMModule.socket.write
  (
    "<iq id='doroster' type='get'><query xmlns='jabber:iq:roster'/></iq>"
  );
}

JabberXM.prototype.getRosterDataSource = function()
{
  return JabberXMModule.iq.roster.datasource;
}

JabberXM.prototype.addRosterGroup = function(group)
{
	JabberXMModule.iq.setGroup(group);
}

JabberXM.prototype.removeRosterGroup = function(group)
{
	JabberXMModule.iq.removeGroup(group);
}

/******* SECTION: PRESENCE *******/

JabberXM.prototype.setPresence = function(show,status)
{
  JabberXMModule.presence.sendPresence(show,status);
}

JabberXM.prototype.sendPresence = function(jid)
{
  JabberXMModule.socket.write('<presence to="' + jid + '"/>');
}

JabberXM.prototype.getPresenceDataSource = function()
{
  return JabberXMModule.presence.presence.datasource;
}

JabberXM.prototype.subscribePresence = function(jid)
{
  JabberXMModule.socket.write('<presence to="'+jid+'" type="subscribe"/>');
}

JabberXM.prototype.subscribedPresence = function(jid)
{
  JabberXMModule.socket.write('<presence to="'+jid+'" type="subscribed"/>');
}

JabberXM.prototype.unsubscribePresence = function(jid)
{
  JabberXMModule.socket.write('<presence to="'+jid+'" type="unsubscribe"/>');
}

JabberXM.prototype.unsubscribedPresence = function(jid)
{
  JabberXMModule.socket.write('<presence to="'+jid+'" type="unsubscribed"/>');
}

JabberXM.prototype.initPresenceRequest = function(window,uri,attributes)
{
  JabberXMModule.presence.presenceRequestWindow=window;
  JabberXMModule.presence.presenceRequestWindowUri=uri;
  JabberXMModule.presence.presenceRequestAttributes=attributes;
}

/******* SECTION: CONTACT *******/

JabberXM.prototype.addContact = function(jid,nickname,group,reason)
{
  JabberXMModule.socket.write
  (
    '<iq id="addContact" type="set">' 
    + '<query xmlns="jabber:iq:roster">'
    + '<item jid="'+jid+'" name="'+nickname+'"><group>'+group+'</group></item>'
    + '</query>'
    + '</iq>'
  );

  JabberXMModule.socket.write
  (
    '<presence to="'+jid+'" type="subscribe"><status>'+reason+'</status></presence>'
  );
}

JabberXM.prototype.removeContact = function(jid,group) // UPDATE nick too?
{
  JabberXMModule.socket.write
  (
    '<iq id="delContact" type="set">' 
    + '<query xmlns="jabber:iq:roster">'
    + '<item ask="subscribe" jid="'+jid+'" name="'+nickname+'" subscription="remove"><group>'+group+'</group></item>'
    + '</query>'
    + '</iq>'
  );
}

JabberXM.prototype.setNickname = function(jid,nickname)
{
}

/******* SECTION: MESSAGE *******/

JabberXM.prototype.setChatHandler = function(chatHandler)
{
  JabberXMModule.message.setChatHandler(chatHandler);
}

JabberXM.prototype.setMessageHandler = function(messageHandler)
{
  JabberXMModule.message.setMessageHandler(messageHandler);
}

JabberXM.prototype.openChat = function(jid)
{
  JabberXMModule.message.openChat(jid);
}

JabberXM.prototype.openMessage = function(jid)
{
  JabberXMModule.message.openMessage(jid);
}

JabberXM.prototype.closeChat = function(jid)
{
  JabberXMModule.message.closeChat(jid);
}

JabberXM.prototype.closeMessage = function(jid)
{
  JabberXMModule.message.closeMessage(jid);
}

JabberXM.prototype.sendChat = function(jid,subject,body,html,x)
{
  if(subject != '' || subject != null)
   subject = '<subject>' + subject + '</subject>';

  if(x == null) x = '';

  JabberXMModule.socket.write
  (
    '<message to="' + jid + '" type="chat">' + 
     subject +
     '<body>' + body + '</body>' +
     '<html xmlns="http://www.w3.org/1999/xhtml">' + html + '</html>' +
     x + 
    '</message>'
  );
}

JabberXM.prototype.sendGroupChat = function(jid,subject,body,html,thread,x)
{
  if(subject != '' || subject != null)
    subject = '<subject>' + subject + '</subject>';

  JabberXMModule.socket.write
  (
    '<message to="' + jid + '" type="groupchat">' + 
    '<thread>' + thread + '</thread>' +
     subject +
    '<body>' + body + '</body>' +
     html +
    '</message>'
  );
}

JabberXM.prototype.sendMessage = function(jid,subject,body,html,x)
{
  if(subject != '')
    subject = '<subject>' + subject + '</subject>';

  if(x == null) x = '';
  JabberXMModule.socket.write
  (
   '<message to="' + jid + '">' + 
    subject +
    '<body>' + body + '</body>' +
    '<html>' + html + '</html>' +
    x +
   '</message>'
  );
}

JabberXM.prototype.initChat = function(window,uri,attributes)
{
   JabberXMModule.message.initChat(window,uri,attributes);
}

JabberXM.prototype.initMessage = function(window,uri,attributes)
{
  JabberXMModule.message.initMessage(window,uri,attributes);
}

/*************************************/
// NAMESPACE

JabberXM.prototype.registerNSHandler = function( namespace, handler )
{
   JabberXMModule.message.registerNSHandler( namespace, handler );
}

JabberXM.prototype.unregisterNSHandler = function( namespace, handler )
{
   JabberXMModule.message.unregisterNSHandler( namespace, handler );
}

/*************************************/
// IQ

JabberXM.prototype.sendFile = function(url)
{
}

JabberXM.prototype.initDoneFile = function(doneWindowUri,attributes)
{
}

JabberXM.prototype.initReceiveFile = function(recieveWindowUri,attributes)
{
}

JabberXM.prototype.search = function(first,last,nickname,email)
{
}

JabberXM.prototype.getSearchDataSource = function()
{
}

JabberXM.prototype.getVCard = function(jid)
{
  JabberXMModule.socket.write('<iq type="get" to="' + jid + 
                              '"><VCARD xmlns="vcard-temp"/></iq>');
}

JabberXM.prototype.initClientVersion = function(version)
{
   initialize(); // HACK!
}

JabberXM.prototype.sendIQ = function(jid,type,attributes,contents) {

  iq="<iq to='" + jid + "' " + attributes + " type='" + type  + "'>" +contents+ "</iq>";
  JabberXMModule.socket.write(iq);
}

/*************************************/
// WEB CONTENT

JabberXM.prototype.setPostHandler = function(postHandler) {
  postHandler = postHandler.wrappedJSObject;
  JabberXMModule.message.postHandler = postHandler;
}

/*************************************/
// CONFERENCE

JabberXM.prototype.getConferenceRoster = function(room)
{
  return JabberXMModule.conferenceRosters[room].roster.datasource;
}

JabberXM.prototype.setConferenceChatHandler = function(room,chatHandler)
{
  JabberXMModule.conferenceChatHandlers[room] = chatHandler.wrappedJSObject.chatHandler;
}

/*************************************/
// WHITEBOARD

JabberXM.prototype.setWhiteboardHandler = function(whiteboardHandler)
{
   whiteboardHandler = whiteboardHandler.wrappedJSObject;

   JabberXMModule.message.whiteboardHandler = whiteboardHandler;
}



/*************************************/

JabberXM.prototype.QueryInterface = function(iid)
{
  if (!iid.equals(CiJabberXM))
    throw Components.results.NS_ERROR_NO_INTERFACE;
  return this;
}



/**********************************************************************************/

JabberXMFactory = new Object();

JabberXMFactory.createInstance = function(outer,iid)
{
  if (outer != null)
    throw Components.results.NS_ERROR_NO_AGGREGATION;

  if (!iid.equals(CiJabberXM) && !iid.equals(nsISupports))
    throw Components.results.NS_ERROR_INVALID_ARG;

  return new JabberXM();
}

/**********************************************************************************/

var JabberXMModule =
{
   isConnected       : false,
   hasReceivedRoster : false,

   username          : '',
   server            : '',

   socket            : null,
   stream            : null,

   message           : null,

   session           : null,

   iq                : new IQ(),	
   presence          : new Presence(),	
   //roster            : new Roster(),

   conferenceRosters      : new Array(),
   conferenceChatHandlers : new Array(),
   conferenceUserNameList : new Array(),

   chatWindows       : new Object(),
   messageWindows    : new Object(),
}

JabberXMModule.registerSelf = function(compMgr,fileSpec,location,type)
{
  dump("*** Registering JabberXM.\n");
    
  compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
  compMgr.registerFactoryLocation(CID,"JabberXM","@jabberxm.mozdev.org;1",fileSpec,location,type);
}

JabberXMModule.unregisterSelf = function(compMgr, fileSpec, location)
{
  compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
  compMgr.unregisterFactoryLocation(CID,fileSpec);
}

JabberXMModule.getClassObject = function(compMgr,cid,iid)
{
  if(cid.equals(CID)) return JabberXMFactory;
    
  if(!iid.equals(Components.interfaces.nsIFactory))
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

  throw Components.results.NS_ERROR_NO_INTERFACE;
}

JabberXMModule.canUnload = function(compMgr)
{
  return true;
}

function NSGetModule(compMgr, fileSpec)
{
  return JabberXMModule;
}

/*
 * Summary:
 *
 *  This handles all the message handling of Jabberzilla for JabberXM.
 */

function Message()
{
  this.windowMediatorService =
         Components.classes["@mozilla.org/appshell/window-mediator;1"].
           getService(Components.interfaces.nsIWindowMediator);

  this.wrappedJSObject = this;
  this.postHandler = null;
  // Array of namespaces each containing an array of handlers for that namespace.
  this.nsHandler = new Array(); 
}

Message.prototype.initChat = function(window,chatWindowUri,attributes) {
  this.chatWindow=window;
  this.chatWindowUri=chatWindowUri;
  this.chatAttributes=attributes;
}

Message.prototype.setChatHandler = function(chatHandler)
{
  this.chathandler = chatHandler;	//update chatHandlerchatHandler to chatHandler
}

Message.prototype.setMessageHandler = function(messageHandler)
{
  this.messagehandler = messageHandler.messageHandler;
}

Message.prototype.handleChat = function(jid,subject,body,html)
{
  var xulWindows = this.windowMediatorService.getEnumerator("jabberzilla:chat",true);
  var hasMoreElements = true;
  var foundWindow;

  while(hasMoreElements)
  {
    if(xulWindows.hasMoreElements())
    {
      var xulWindow = xulWindows.getNext();
      xulWindow.QueryInterface(Components.interfaces.nsIDOMWindow);
      xulWindow = xulWindow;

      if(xulWindow.name==jid)
      {
        foundWindow = xulWindow;
        break;
      }
    }
    else
    { 
			var firstMessage = new Array (jid,subject,body,html);
			foundWindow = this.chatWindow.openDialog('chrome://jabberzilla/content/chat.xul',jid,'chrome,all,dialog=no',"","",firstMessage);

      foundWindow.name = jid;
      hasMoreElements = false;
    }
  }

  foundWindow.getAttention();
  if (hasMoreElements)
		{ foundWindow.incomingMessage(jid,subject,body,html); }
}

Message.prototype.openChat = function(jid)
{
  var xulWindows = this.windowMediatorService.getXULWindowEnumerator("navigator:browser");
  var hasMoreElements = true;

  while(hasMoreElements)
  {
    if(xulWindows.hasMoreElements())
    {
      var xulWindow = xulWindows.getNext();
      xulWindow.QueryInterface(Components.interfaces.nsIXULWindow);

      if(xulWindow.name==jid)
        break;
    }
    else
    {
      this.chatWindow.openDialog('chrome://jabberzilla/content/chat.xul',jid,'chrome,all,dialog=no');
			hasMoreElements = false;
    }
  }
}

Message.prototype.closeChat = function(jid)
{
  JabberXMModule.chatWindows[jid] = null;
}

Message.prototype.openMessage = function(jid)
{
  if(JabberXMModule.messageWindows[jid] == null)
  {
    JabberXMModule.messageWindows[jid] = 
      this.messageWindow.open(this.messageWindowUri,jid,this.messageAttributes);

    JabberXMModule.messageWindows[jid].messageHandler = this.messageHandler;
  }
}

Message.prototype.messageChat = function(jid)
{
  JabberXMModule.chatWindows[jid] = null;
}

Message.prototype.packetMessage = function(packet)
{
  var messageType = '';

  switch( packet.attribs["type"] )
  {
    case 'chat':
      messageType = 'chat';
      break;
    case 'groupchat':
      messageType = 'groupchat';
      break;
    case 'error':
      messageType = 'error';
      break;
    default:
      messageType = 'normal';
  }

  var subject=''; var body=''; var thread=''; var html=''; var w='';

  var calledHandler = false;

  for(var i = 0; i < packet.children.length; i++)
  {
    var child = packet.children[i];
    switch( child.name )
    {
      case "subject":
        subject = child.cdata;
        break;

      case "body":
        body = child.cdata;
        break;

      case "thread":
        thread = child.cdata;
        break;

      case "html":
        var currentData = JabberXMModule.socket.getData();
        var beginning = currentData.search("<html");
        html = currentData.substring(beginning);
        var end = html.search("</html>");
        html = html.substring(0,end+7);
        break;

      case "w":
        var currentData = JabberXMModule.socket.getData();
        var beginning = currentData.search("<w");
        w = currentData.substring(beginning);
        var end = w.search("</w>");
        w = w.substring(0,end+4);
        break;

      case "x":
        var xmlns = child.attribs["xmlns"];
        if( this.nsHandler[ xmlns ] )
        {
          var handler = this.nsHandler[ xmlns ];
          for( var i = 0; i < handler.length; i++ )
          {
            handler[i].handleMessage( packet );
            calledHandler = true;
          }
        }
        else
        {
          dump( "No handler registered for namespace " + xmlns + "." );
        }
        break;

      default:
        dump( "unknown message element: " + child.name );
    }
  }

  if(calledHandler)
    return;

  var fulljid =packet.attribs["from"];
  var jid = packet.attribs["from"] + "/";
  jid = jid.substr( 0, jid.indexOf( "/" ) );

  if(messageType=='normal' || messageType=='chat')
  {
    //this.handleMessage(jid,subject,body);
    this.handleChat(jid,subject,body,html);
  }
  else if(messageType=='groupchat')
  {
    if(w != '')
    {
      name = JabberXMModule.conferenceUserNameList[fulljid];
      this.whiteboardHandler.handleWhiteboard(name,body,w);
    }
    else if(JabberXMModule.conferenceChatHandlers[jid] != null)
    {
      name = JabberXMModule.conferenceUserNameList[fulljid];
      if(name==null)
       name='';

      JabberXMModule.conferenceChatHandlers[jid](name,subject,body,html);
    }
    else if(this.postHandler != null)
    {
      this.postHandler.handlePost(jid,subject,body,thread);
    }
  }
  else
  {

  }
}

Message.prototype.registerNSHandler = function(namespace,handler)
{
  var newHandler = handler.wrappedJSObject;
  if( this.nsHandler[ namespace ] )
  {
    if( this.nsHandler[ namespace ][ newHandler ] )
    {
      dump( "Handler already registered." );
      return false;
    }
    else
      this.nsHandler[ namespace ].push( newHandler );
  }
  else
  {
    this.nsHandler[ namespace ] = new Array( newHandler );
  }
  return true;
}

Message.prototype.unregisterNSHandler = function(namespace,handler)
{
  var oldHandler = handler.wrappedJSObject;
  if( this.nsHandler[ namespace ] )
  {
    for( var i = 0; i < this.nsHandler[ namespace ].length; i++ )
    {
      if( this.nsHandler[ namespace ][ i ] == oldHandler )
      {
        this.nsHandler[ namespace ].splice( i, 1 );
        if( this.nsHandler[ namespace ].length == 0 )
        {
         //TODO: delete the namespace entry - something like this:
         //this.nsHandler[ namespace ] = undefined;
        }
        return true;
      }
    }
  }
  return false;
}

/*
 * Summary:
 *
 *  This handles all the presence handling of Jabberzilla for JabberXM.
 */

function Presence()
{
  this.presence = new DataSequence('http://jabberxm.mozdev.org/rdf#groups');	
  this.showResource  = 'http://jabberxm.mozdev.org/rdf#show';
  this.statusResource = 'http://jabberxm.mozdev.org/rdf#status';
  this.presenceLoaded = false;
}

Presence.prototype.sendPresence = function(show, status)
{
  if (show == 'available')
    JabberXMModule.socket.write('<presence/>');

  else
  {
    JabberXMModule.socket.write
    (
      '<presence><show>' + 
       show  + '</show><status>' + 
       status + '</status></presence>'
    );
  }
}

Presence.prototype.changeContactPresence = function(jid,res,show,status)
{
  this.presenceLoaded = true;

  if(res!='')
    var user = this.presence.rdf.GetResource(jid+'/'+res);
  else
    var user = this.presence.rdf.GetResource(jid);

  target = 
    this.presence.datasource.GetTarget
    (
      user,
      this.presence.rdf.GetResource(this.showResource), 
      true
    );
  if(target==null)
  {

		this.presence.datasource.Assert
    (
      user,
      this.presence.rdf.GetResource(this.showResource),
      this.presence.rdf.GetLiteral(show),
      true
    );

  }
  else if ( JabberXMModule.hasReceivedRoster )
  {

		this.presence.datasource.Change
    (
      user,
      this.presence.rdf.GetResource(this.showResource),
      target,
      this.presence.rdf.GetLiteral(show)
    );

  }

  target = 
    this.presence.datasource.GetTarget
    (
      user,
      this.presence.rdf.GetResource(this.statusResource),
      true
    );

  if ( target == null )
  {
    this.presence.datasource.Assert
    (
      user,
      this.presence.rdf.GetResource(this.statusResource),
      this.presence.rdf.GetLiteral(status),
      true
    );
  }
  else if ( JabberXMModule.hasReceivedRoster )
  {
    this.presence.datasource.Change
    (
      user,
      this.presence.rdf.GetResource(this.statusResource),
      target,
      this.presence.rdf.GetLiteral(status)
    );
  }

  this.presence.sequence.AppendElement(user);

  if(res!='') {
    res='';
    this.changeContactPresence(jid,res,show,status);
  }
	//change the content in rdf:http://jabberxm.mozdev.org/rdf#rosteritems
	JabberXMModule.iq.changeRosteritems(jid,show,status);
}

/*
  this.presence.datasource.Unassert
  (
    user,
    this.presence.rdf.GetResource(this.showResource),
    this.presence.rdf.GetLiteral(show)
  );
*/

Presence.prototype.packetPresence = function(packet)
{
  // Get the JID and Resource (hack)
  p = '';  q = 0;

  if(packet.attribs["type"]=='error')
  {}
  else if(packet.attribs["type"]=='subscribe')
  {
    jid = packet.attribs["from"]; status = '';

    for(var i = 0; i < packet.children.length; i++){
      if(packet.children[i].name == "status")
        status = packet.children[i].cdata;
    }
    this.presenceRequestWindow.open
    (this.presenceRequestWindowUri,jid,this.presenceRequestAttributes);
  }

  else if (packet.attribs["from"]!=null)
  {
    from = packet.attribs["from"];

    // Hack off resource, except for Agents

    if (from.search('icq')!=-1 && from.search('@icq')==-1)
    {
      jid = from; res = '';
    }
    else if (from.search('aim')!=-1 && from.search('@aim')==-1)
    {
      jid = from; res = '';
    }
    else if (from.search('msn')!=-1 && from.search('@msn')==-1)
    {
      jid = from; res = '';
    }
    else if (from.search('yahoo')!=-1 && from.search('@yahoo')==-1)
    {
      jid = from; res = '';
    }
    else
    {
      from=packet.attribs["from"] + '/';

      while(p != '/')
      {
        p=from[q++];
      }

      jid=(from.slice(0,q-1));

      res=(from.slice(q,from.length-1));
    }
  }

  // Get the Priority
  for(var i = 0; i < packet.children.length; i++)
  {
    if(packet.children[i].name == "priority")
    {
      priority = packet.children[i].cdata;
    }
  }

  // Default
  show = 'available';

  if(packet.attribs["type"]=='error')
  {
    show = 'unknown';
  }

  if (packet.attribs["type"]=='unavailable')
  {
    show = 'unavailable';
  }

  for(i=0;i<packet.children.length;i++)
  {
    if(packet.children[i].name=="show")
    {
      if (packet.children[i].cdata != '')
        show = packet.children[i].cdata;
    }
  }

  if(show == 'available')
    status = 'Online';
  else
    status = '';

  for(var i = 0; i < packet.children.length; i++)
  {
    if(packet.children[i].name == "status")
    {
      status = packet.children[i].cdata;
    }
  }
  this.changeContactPresence(jid,res,show,status);
}

/*
 * Summary:
 *
 *  This handles all the IQ handling of Jabberzilla for JabberXM, and integrates roster code.
 */

function IQ()
{
  this.rosteritemsResourceURI = 'http://jabberxm.mozdev.org/rdf#rosteritems';
  this.labelResourceURI = 'http://jabberxm.mozdev.org/rdf#label';
  this.typeResourceURI = 'http://jabberxm.mozdev.org/rdf#type';

  // Used for initializing presence
  this.showResourceURI  = 'http://jabberxm.mozdev.org/rdf#show';
  this.statusResourceURI = 'http://jabberxm.mozdev.org/rdf#status';

  this.roster = new DataSequence(this.rosteritemsResourceURI);	
  this.groups = new Object(); // associative array
}

IQ.prototype.packetIQ = function(packet)
{
  if(packet.attribs['type'] == 'result' && packet.attribs['id'] == 'register')
  {
    this.jabberXM.setAuthenticate(this.jabberXM.getUsername(),this.jabberXM.getPassword(),'Jabberzilla','0');
    this.jabberXM.authenticate('normal',this.jabberXM.getPassword());
  }
  else if(packet.attribs['type'] == 'error' && packet.attribs['id'] == 'register')
  {
    alert('"'+ this.jabberXM.getUsername() + '" already exists. Please try another username.');
    this.jabberXM.disconnect();
  }
  else
  {
    for(var i = 0; i < packet.children.length; i++)		//add "break" elements in this cycle
    {
      switch(packet.children[i].name)
      {
        case 'vCard':
        {
          var vCard = packet.children[i];

          for (var x=0; x < vCard.children.length; x++) // Works, with lag
          {
            if (vCard.children[x].name=='EMAIL')
            {
              var Email =  (vCard.children[x].cdata);
              // window._content.location = ( 'mailto:' + Email + '?Subject=File Transfer' );
            }
          }
		  		break;
        }
        case 'query':
        {
          switch(packet.children[i].attribs["xmlns"])
          {
            case 'jabber:iq:roster':
            {
              this.packetRoster(packet.children[i]);
			  			break;
            }
            case 'jabber:iq:agents':
            {
              dump('Agents listing received.');
			  			break;
            }
            case 'jabber:iq:agent':
            {
              dump('Agent properties received.');
			  			break;
            }
            case 'jabber:iq:oob':
            {
              var oob = packet.children[i];
              dump('Out of band received.');
              // window._content.location = packet.children[0].children[0].cdata;
              // writeIQ(packet.attribs["from"],"result");
			  			break;
            }
            case 'jabber:iq:time':
            {
              var time = packet.children[i];
			  			break;
            }
            case 'jabber:iq:version':
            {
              var version = packet.children[i];
			  			break;
            }
            default:
              dump('Unknown query received.');
          }
		  		break;
        }
        case 'user':
          break;
        case 'conference':
          var jid = packet.attribs["from"];
      }
    }
  }
}

/**********************************************************************************/
// Below are functions for handling the roster

IQ.prototype.packetRoster = function(packet)
{
  JabberXMModule.hasReceivedRoster = true;
  for(var h = 0; h < packet.children.length; h++)
  {
    var user = new Object();

    switch(packet.children[h].name)
    {
      case 'item':
      {
        user.jid = packet.children[h].attribs["jid"];
        user.jid = user.jid.toLowerCase();
        user.nickname = packet.children[h].attribs["name"];

        user = this.changeTransportNames(user);
        child = packet.children[h];

        for(var j = 0; j < child.children.length; j++)
        {
          if(child.children[j].name == 'group')
            user.group = child.children[j].cdata;
        }

        if(user.group==null)
          user.group = 'Default';

        this.setGroup(user.group);

        subscription = packet.children[h].attribs["subscription"];

        if(subscription=='to' || subscription=='both')
          this.setUser(user.jid,user.nickname,'unavailable','',user.group);
        else
          this.setUser(user.jid,user.nickname,'unknown','unknown',user.group);
      }
      case 'user':
      {
        user.jid = packet.children[h].attribs["jid"];
        user.nickname = packet.children[h].attribs["name"];
        user.group = "Group";

        if(packet.children[h].attribs["type"]=="remove")
          this.removeUser(user.jid,user.group);
        else
        {
          //this.setGroup(user.group);
          //this.setUser(user.jid,user.nickname,'available','Online',user.group);
          //JabberXMModule.conferenceUserNameList[user.jid] = user.nickname;
        }
      }
      case 'conference':
      {
        for(j=0;j<packet.children[h].children.length;j++)
        {
          //users=packet.children[h].children;

          //user.jid = users[j].attribs["jid"];
          //user.nickname = users[j].attribs["name"];
          //user.group = "Group";

          //this.setGroup(user.group);
          //this.setUser(user.jid,user.nickname,'available','Online',user.group);
          //JabberXMModule.conferenceUserNameList[user.jid] = user.nickname;
        }
      }
    }
  }
}

IQ.prototype.setUser = function(jid,nickname,show,status,group)
{
  var contactResource = this.roster.rdf.GetResource(jid);

  this.roster.datasource.Assert
  (
    contactResource,
    this.roster.rdf.GetResource(this.labelResourceURI),
    this.roster.rdf.GetLiteral(nickname),
    true
  );

  this.roster.datasource.Assert
  (
    contactResource,
    this.roster.rdf.GetResource(this.typeResourceURI),
    this.roster.rdf.GetLiteral('contact'),
    true
  );

  this.roster.datasource.Assert
  (
    contactResource,
    this.roster.rdf.GetResource(this.showResourceURI),
    this.roster.rdf.GetLiteral('unavailable'),
    true
  );

  this.roster.datasource.Assert
  (
    contactResource,
    this.roster.rdf.GetResource(this.statusResourceURI),
    this.roster.rdf.GetLiteral(''),
    true
  );

  this.groups[group].AppendElement(contactResource);
	
  //JabberXMModule.presence.changeContactPresence(jid,'',show,status);
}

IQ.prototype.removeUser = function(jid,group)
{
  var contactResource = this.roster.rdf.GetResource(jid);
  this.groups[group].RemoveElement(contactResource,true);
}

IQ.prototype.setGroup = function(group)
{
  if (this.groups[group]==null)
  {
    // Moz bug 105759
    var groupResource = this.roster.rdf.GetResource('chrome://jabberzilla/content/main.xul#' + group);

    this.roster.datasource.Assert
    (
      groupResource,
      this.roster.rdf.GetResource(this.labelResourceURI),
      this.roster.rdf.GetLiteral(group),
      true
    );

    this.roster.datasource.Assert
    (
      groupResource,
      this.roster.rdf.GetResource(this.labelResourceURI),
      this.roster.rdf.GetLiteral(group),
      true
    );

    this.roster.datasource.Assert
    (
      groupResource,
      this.roster.rdf.GetResource(this.typeResourceURI),
      this.roster.rdf.GetLiteral('group'),
      true
    );

    contactsAnonymousResource = this.roster.rdf.GetAnonymousResource();

    this.roster.datasource.Assert
    (
      groupResource,
      this.roster.rdf.GetResource(this.rosteritemsResourceURI),
      contactsAnonymousResource,
      true
    );

    this.groups[group] = 
      this.roster.rdfc.MakeSeq(this.roster.datasource,contactsAnonymousResource);

    this.roster.sequence.AppendElement(groupResource);
  }
}

IQ.prototype.removeGroup = function(group)
{
  if (this.groups[group] != null)
  {
    //Remove Group Resource
  }
}

// frozen - eric@mozdev.org - 2001.10.16
IQ.prototype.changeTransportNames = function(user)
{
  if(user.jid.search('icq') != -1 && user.jid.search('@icq') == -1) {
    user.nickname = ('ICQ Network');
  }
  else if (user.jid.search('aim') != -1 && user.jid.search('@aim') == -1) {
    user.nickname = ('AIM Network');
  }
  else if (user.jid.search('msn') != -1 && user.jid.search('@msn') == -1) {
    user.nickname = ('MSN Network');
  }
  else if (user.jid.search('yahoo') != -1 && user.jid.search('@yahoo') == -1) {
    user.nickname = ('Yahoo Network');
  }
  return user;
}

//additional function by Patton@sun Dec 26 2002
IQ.prototype.changeRosteritems = function(jid,show,status)
{
	var contactResource = this.roster.rdf.GetResource(jid);
	
  target = 
    this.roster.datasource.GetTarget
    (
      contactResource,
      this.roster.rdf.GetResource(this.showResourceURI), 
      true
    );
	this.roster.datasource.Change
  (
    contactResource,
    this.roster.rdf.GetResource(this.showResourceURI),
    target,
    this.roster.rdf.GetLiteral(show)
  );
	
  target = 
    this.roster.datasource.GetTarget
    (
      contactResource,
      this.roster.rdf.GetResource(this.statusResourceURI), 
      true
    );
	this.roster.datasource.Change
  (
    contactResource,
    this.roster.rdf.GetResource(this.statusResourceURI),
    target,
    this.roster.rdf.GetLiteral(status)
  );
}

/**********************************************************************************/
// This is an object for creating a list of data in RDF.

function DataSequence(uri)
{
  this.rdf = Components.classes['@mozilla.org/rdf/rdf-service;1'].getService(Components.interfaces.nsIRDFService);

  this.datasource = Components.classes['@mozilla.org/rdf/datasource;1?name=in-memory-datasource'].createInstance(Components.interfaces.nsIRDFDataSource);

  this.root = this.rdf.GetResource('urn:root');
  this.anon = this.rdf.GetAnonymousResource();

  this.datasource.Assert(this.root,this.rdf.GetResource(uri),this.anon,true);

  this.rdfc = Components.classes['@mozilla.org/rdf/container-utils;1'].getService(Components.interfaces.nsIRDFContainerUtils);

  this.sequence = this.rdfc.MakeSeq(this.datasource,this.anon);

  this.rdfs = Components.classes['@mozilla.org/rdf/container;1'].getService(Components.interfaces.nsIRDFContainer);
}