/* 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * Contributor(s): Eric Murphy,    ericmurphy@jabber.org
 *                 Jeremie Miller, jeremie@jabber.org
 * 2002.03.02
 *
 *****************************************************************************/

const CONTRACTID  = "@jabberxm.mozdev.org/sax;1";
const CID         = Components.ID("{b4cdcfae-1dd1-11b2-9c25-9b5d19103745}");
const CiJabberXM  = Components.interfaces.JabberXMSax;
const nsISupports = Components.interfaces.nsISupports;

function streamElement(name)
{
   this.name     = name;
   this.attribs  = new Array();
   this.children = new Array();
   this.cdata    = new String();
}

function streamDocumentHandler(nodes)
{
   this.empty = false;
   this.root  = false;
   this.x     = false;
   this.nodes = nodes;
}

streamDocumentHandler.prototype.startDocument = function(){}

streamDocumentHandler.prototype.endDocument = function(){}

streamDocumentHandler.prototype.startElement = function(name, attribs)
{
   var x = new streamElement(name);
   x.attribs = attribs;
   x.parent = this.x;
   if(this.x != false)
   {
      this.x.children[this.x.children.length] = x;
   }
   this.x = x;

   if(this.root == false)
   { // root stream element
      this.root = x;
      this.nodes(x);
   }
}

streamDocumentHandler.prototype.endElement = function(name)
{
   var x = this.x;
   this.x = this.x.parent;
   if(this.x == false)
   { // stream closed
   }
   if(this.x == this.root)
   { // top level element
      this.nodes(x);
   }
}

streamDocumentHandler.prototype.characters = function(data)
{
   this.x.cdata += data;
}

/*****************************************************************************/

function JabberXMSax()
{
   this.doc = false;
   this.handler = false;
   this.str = "";
   this.cname = new Array();
   this.curr = 0;
   this.wrappedJSObject = this;
}

JabberXMSax.prototype.setDocumentHandler = function(handler)
{
   handlers = handler.wrappedJSObject;
   this.handler = new streamDocumentHandler(handlers.packetHandler);
}

JabberXMSax.prototype.parse = function(data)
{
   this.str += data;

   if(this.handler == false)
   {  // code exceptions :)
      return;
   }

   if(!this.doc)
   {  // strip prolog
      var start = this.str.indexOf("<");
      if(this.str.substring(start,start + 3) == "<?x" || 
         this.str.substring(start,start + 3) == "<?X" )
      {
         var close = this.str.indexOf("?>");
         if(close == -1)
         {
            return;
         }
         this.str = this.str.substring(close + 2,this.str.length);
      }
      this.handler.startDocument();
      this.doc = true;
   }

   // keep circling and eating the str
   while(1)
   {
      if(this.str.length == 0)
      {
         return;
      }

      // check for closes first
      var eclose = this.str.indexOf("</" + this.cname[this.curr] + ">");
      if(eclose == 0)
      {
         this.str = this.str.substring(this.cname[this.curr].length + 3,
                                       this.str.length);
         this.handler.endElement(this.cname[this.curr])
         this.curr--;
         if(this.curr == 0)
         {
            this.doc = false;
            this.handler.endDocument();
            return;
         }
         continue;
      }

      // check last for tags
      var estart = this.str.indexOf("<");
      if(estart == 0)
      {  // new element
         var close = this.str.indexOf(">");
         if(close == -1)
         {
            return;
         }
         var empty = (this.str.substring(close - 1,close) == "/");
         if(empty)
         {
            var starttag = this.str.substring(1,close - 1);
         }else{
            var starttag = this.str.substring(1,close);
         }
         var nextspace = starttag.indexOf(" ");
         var attribs = new String();
         var name = new String();
         if(nextspace != -1)
         {
            name = starttag.substring(0,nextspace);
            attribs = starttag.substring(nextspace + 1,starttag.length);
         }else{
            name = starttag;
         }

         this.handler.startElement(name, this.attribution(attribs));

         if(empty)
         {
            this.handler.endElement(name);
         }else{
            this.curr++;
            this.cname[this.curr] = name;
         }

         this.str = this.str.substring(close + 1,this.str.length);
         continue;
      }

      // leftovers are cdata
      if(estart == -1)
      {
         this.handler.characters(this.entityCheck(this.str));
         this.str = "";
      }else{
         this.handler.characters(this.entityCheck(this.str.substring(0,estart)));
         this.str = this.str.substring(estart,this.str.length);
      }
   }
}

JabberXMSax.prototype.attribution = function(str)
{
   var attribs = new Array();
   var ids = new Number();
   var eq = id1 = id2 = nextid = val = key = new String();

   while(1)
   {
      eq = str.indexOf("=");
      if(str.length == 0 || eq == -1)
      {
         return attribs;
      }

      id1 = str.indexOf("'");
      id2 = str.indexOf('"');
      if((id1 < id2 && id1 != -1) || id2 == -1)
      {
         ids = id1;
         id = "'";
      }
      if((id2 < id1 || id1 == -1) && id2 != -1)
      {
         ids = id2;
         id = '"';
      }

      nextid = str.indexOf(id,ids + 1);
      val = str.substring(ids + 1,nextid);
      key = str.substring(0,eq);

      // strip whitespace
      ws = key.split("\n");
      key = ws.join("");
      ws = key.split(" ");
      key = ws.join("");
      ws = key.split("\t");
      key = ws.join("");

      attribs[key] = this.entityCheck(val);
      str = str.substring(nextid + 1,str.length);
   }
   return attribs;
}

JabberXMSax.prototype.entityCheck = function(str)
{
   var A = new Array();

   A = str.split("&lt;");
   str = A.join("<");
   A = str.split("&gt;");
   str = A.join(">");
   A = str.split("&quot;");
   str = A.join("\"");
   A = str.split("&apos;");
   str = A.join("\'");
   A = str.split("&amp;");
   str = A.join("&");

   return str;
}

/*****************************************************************************/

JabberXMSax.prototype.QueryInterface = function(iid)
{
  if (!iid.equals(CiJabberXM))
    throw Components.results.NS_ERROR_NO_INTERFACE;
  return this;
}

/*****************************************************************************/

JabberXMFactory = new Object();

JabberXMFactory.createInstance = function(outer,iid)
{
  if (outer != null)
    throw Components.results.NS_ERROR_NO_AGGREGATION;

  if (!iid.equals(CiJabberXM) && !iid.equals(nsISupports))
    throw Components.results.NS_ERROR_INVALID_ARG;

  return new JabberXMSax();
}


/*****************************************************************************/

var JabberXMSaxModule = new Object();

JabberXMSaxModule.registerSelf = function(compMgr,fileSpec,location,type)
{
  dump("*** Registering JabberXMSax.\n");
    
  compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
  compMgr.registerFactoryLocation(CID,"JabberXMSax",CONTRACTID,fileSpec,location,type);
}

JabberXMSaxModule.unregisterSelf = function(compMgr, fileSpec, location)
{
  compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
  compMgr.unregisterFactoryLocation(CID,fileSpec);
}

JabberXMSaxModule.getClassObject = function(compMgr,cid,iid)
{
  if(cid.equals(CID)) return JabberXMFactory;
    
  if(!iid.equals(Components.interfaces.nsIFactory))
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

  throw Components.results.NS_ERROR_NO_INTERFACE;
}

JabberXMSaxModule.canUnload = function(compMgr)
{
  return true;
}

function NSGetModule(compMgr, fileSpec)
{
  return JabberXMSaxModule;
}