/* 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 *
 * Contributor(s): Eric Murphy, ericmurphy@jabber.org
 *                 Rob Ginda,   rginda@netscape.com
 * 2002.03.02
 *
 *****************************************************************************/

const CONTRACTID  = "@jabberxm.mozdev.org/socket;1";
const CID         = Components.ID("{a4cdcfae-1dd1-11b2-9c25-9b5d19103743}");
const CiJabberXM  = Components.interfaces.JabberXMSocket;
const nsISupports = Components.interfaces.nsISupports;

var StreamProvider = new Object();

StreamProvider.pendingData    = "";
StreamProvider.isBlocked      = true;
StreamProvider.onStartRequest = function(request, ctxt){}
StreamProvider.onStopRequest  = function(request, ctxt, status){}

StreamProvider.onDataWritable = 
function (request, ctxt, ostream, offset, count)
{
    if (!this.pendingData)
    {
        this.isBlocked = true;
        throw Components.results.NS_BASE_STREAM_WOULD_BLOCK;
    }

    var len = ostream.write (this.pendingData, this.pendingData.length);
    this.pendingData = this.pendingData.substr(len);
}

/*****************************************************************************/

// implement isConnected into interface and this code.
// implement view stream code mechanism

function StreamListener(){}

StreamListener.prototype.onStartRequest = function (request, ctxt){}

StreamListener.prototype.onStopRequest = 
function (request, ctxt, status, errorMsg)
{
    ctxt = ctxt.wrappedJSObject;
    JabberXMSocketModule.isConnected = false;
}

StreamListener.prototype.onDataAvailable = 
function (request, ctxt, inStr, sourceOffset, count)
{
    ctxt = ctxt.wrappedJSObject; 
    if(!ctxt._inputStream)
        ctxt._inputStream = ctxt.toScriptableInputStream(inStr);

    ctxt.startAsyncRead(ctxt.read());
}

/*****************************************************************************/

function JabberXMSocket()
{
    var sockServiceClass =
        Components.classesByID["{c07e81e0-ef12-11d2-92b6-00105a1b0d64}"];
    
    if (!sockServiceClass)
        throw ("Couldn't get socket service class.");
    
    var sockService = sockServiceClass.getService();
    if (!sockService)
        throw ("Couldn't get socket service.");

    this._sockService = sockService.QueryInterface
        (Components.interfaces.nsISocketTransportService);

    this.data = '';

    this.wrappedJSObject = this;
}

JabberXMSocket.prototype.stream = function(stream)
{
   this.stream = stream.wrappedJSObject;
}

JabberXMSocket.prototype.setViewStream = function(viewStream)
{
   this.viewStream = viewStream.wrappedJSObject;
}

JabberXMSocket.prototype.process = function(data)
{
   this.data = data;
   dump("\nXXXXXXXXXXXX  Received from Server: \n"+data+"\n");

   try{this.viewStream.receiveToStream(data);}
   catch(e){}

   try{
      this.stream.parse(data);
   }
   catch(e){
      dump('ERROR ' + e);
   }
}

JabberXMSocket.prototype.getData = function()
{
    return this.data;
}

JabberXMSocket.prototype.initAsyncRead = function()
{
    this.startAsyncRead = function(handler)
    {
        this.startAsyncRead = handler;
        this._channel.asyncRead (new StreamListener(), this, 0, -1, 0);
    }

    this.startAsyncRead(this.process);
}

JabberXMSocket.prototype.toScriptableInputStream = function(i)
{
    var si = Components.classes["@mozilla.org/scriptableinputstream;1"];
    
    si = si.createInstance();
    si = si.QueryInterface(Components.interfaces.nsIScriptableInputStream);
    si.init(i);

    return si;
}

JabberXMSocket.prototype.open = function(socketTypeCount, socketTypes, host, port, proxyInfo)
{
    if (typeof tcp_flag == "undefined")
               tcp_flag = false;

    this.host = host.toLowerCase();
    this.port = port;

    if(socketTypes[0]=="normal") {
      this._channel = this._sockService.createTransport(host, port, null, -1, 0, 0);
    }
    else {
      this._channel = this._sockService.createTransportOfTypes(socketTypeCount, socketTypes, host, port, null, -1, 0, 0);
    }

    if (!this._channel)
        throw ("Error opening channel.");

    this._write_req =
        this._channel.asyncWrite (StreamProvider, null, 0, -1, 0);

    this.connectDate = new Date();
    JabberXMSocketModule.isConnected = true;

    return JabberXMSocketModule.isConnected;
}

JabberXMSocket.prototype.close = function()
{
    if (JabberXMSocketModule.isConnected) {
        JabberXMSocketModule.isConnected = false;
        this._inputStream.close();
    }
}

JabberXMSocket.prototype.write = function(str)
{
		dump("\nXXXXXXXXXXXX  Send to Server: \n"+str+"\n");

   try{this.viewStream.sendToStream(str);}
   catch(e){}

   if (!JabberXMSocketModule.isConnected)
       throw "Not Connected.";

   this.asyncWrite(str);
}

JabberXMSocket.prototype.asyncWrite = function(str)
{
    StreamProvider.pendingData += str;

    if(StreamProvider.isBlocked)
    {
        this._write_req.resume();
        StreamProvider.isBlocked = false;
    }
}

JabberXMSocket.prototype.read = function(timeout)
{
    if (!JabberXMSocketModule.isConnected)
        throw "Not Connected.";

    var rv, av;

    try
    {
        av = this._inputStream.available();
        if (av)
            rv = this._inputStream.read (av);
        else
            rv = "";
    }
    catch (ex)
    {
        dump ("*** Caught " + ex + " while reading.")
        if (typeof ex != "undefined") {
            JabberXMSocketModule.isConnected = false;
            throw (ex);
        } else {
            rv = "";
        }
    }  
    return rv;
}

/*****************************************************************************/

JabberXMSocket.prototype.QueryInterface = function(iid)
{
  if (!iid.equals(CiJabberXM))
    throw Components.results.NS_ERROR_NO_INTERFACE;
  return this;
}

/*****************************************************************************/

JabberXMFactory = new Object();

JabberXMFactory.createInstance = function(outer,iid)
{
  if (outer != null)
    throw Components.results.NS_ERROR_NO_AGGREGATION;

  if (!iid.equals(CiJabberXM) && !iid.equals(nsISupports))
    throw Components.results.NS_ERROR_INVALID_ARG;

  return new JabberXMSocket();
}


/*****************************************************************************/

var JabberXMSocketModule = new Object();

JabberXMSocketModule.registerSelf = function(compMgr,fileSpec,location,type)
{
  dump("*** Registering JabberXMSocket.\n");
    
  compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
  compMgr.registerFactoryLocation(CID,"JabberXMSocket",CONTRACTID,fileSpec,location,type);
}

JabberXMSocketModule.unregisterSelf = function(compMgr, fileSpec, location)
{
  compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
  compMgr.unregisterFactoryLocation(CID,fileSpec);
}

JabberXMSocketModule.getClassObject = function(compMgr,cid,iid)
{
  if(cid.equals(CID)) return JabberXMFactory;
    
  if(!iid.equals(Components.interfaces.nsIFactory))
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

  throw Components.results.NS_ERROR_NO_INTERFACE;
}

JabberXMSocketModule.canUnload = function(compMgr)
{
  return true;
}

function NSGetModule(compMgr, fileSpec)
{
  return JabberXMSocketModule;
}
